/////////////////CODE General Information//////////////////////////////////////////////////////////////////////
// SHYPROM-Smart HYdraulic PROperties Meter
//Information on the code output:
//01. Capacitive Sensor, Top Ring: analogic value;
//02. Capacitive Sensor, Top Ring: voltage (Volts); 
//03. Volumetric Water Content Top Ring (cm3/cm3) optional;                          
//04. Capacitive Sensor, Middle Ring: analogic value;
//05. Capacitive Sensor, Middle Ring: voltage (Volts);
//06. Volumetric Water Content Middle Ring (cm3/cm3) optional;                       
//07. Capacitive Sensor, Bottom Ring: analogic value;
//08. Capacitive Sensor, Bottom Ring: voltage (Volts); 
//09. Volumetric Water Content Bottom Ring (cm3/cm3) optional;                       
//10. Temperature (DS18B20) module: temperature (°C);
//11. Pressure Transducer#1 (MPX5100DP) module: pressure/tension (kPa) optional;
//12. Pressure Transducer#1 (MPX5100DP) module: voltage (volts);
//13. Pressure Transducer#1 (MPX5100DP) module: pressure/tension (cm H2O) optional;
//14. Pressure Transducer#2 (MPX5100DP) module: pressure/tension (kPa) optional; 
//15. Pressure Transducer#2 (MPX5100DP) module: voltage (volts);
//16. Pressure Transducer#2 (MPX5100DP) module: pressure/tension (cm H2O) optional;
////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////// Your GPRS and SIM Card credentials (leave empty, if not needed)//////////////////////////
const char apn[]      = "TM"; // APN (example: TM for a Things mobile SIM)
const char gprsUser[] = ""; // GPRS User
const char gprsPass[] = ""; // GPRS Password
const char simPIN[]   = ""; // SIM card PIN (leave empty, if not defined)
///////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////// Server details/////////////////////////////////////////////////////
const char server[] = "WRITE HERE YOUR CLOUD URL"; // domain name: for e.g.: api.thingspeak.com,
const char resource[] = "/update?";         // for e.g.: update? for a thingspeak channel   

const int  port = 80;                             // server port number
String apiKeyValue = "WRITE HERE YOUR APY KEY";  // Your apikey 

///////////////////////////////////////Modem pins/////////////////////////////
#define MODEM_RST            5
#define MODEM_PWKEY          4
#define MODEM_POWER_ON       23
#define MODEM_TX             27
#define MODEM_RX             26
#define I2C_SDA              21
#define I2C_SCL              22
////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////Include Libraries//////////////////////
#define SerialMon Serial
#define SerialAT Serial1

////////////////////////////// Configure TinyGSM library////////////////////
#define TINY_GSM_MODEM_SIM800      // Modem is SIM800
#define TINY_GSM_RX_BUFFER   1024  // Set RX buffer to 1Kb

#include <Wire.h>
#include <TinyGsmClient.h>

#ifdef DUMP_AT_COMMANDS
  #include <StreamDebugger.h>
  StreamDebugger debugger(SerialAT, SerialMon);
  TinyGsm modem(debugger);
#else
  TinyGsm modem(SerialAT);
#endif

//////////////////////////////// Configure SD CARD libraries and Special PINS////////////////////
// Libraries for SD card
#include "FS.h"
#include "SD.h"
#include <SPI.h>
//
//////////////////////////////////////////////
SPIClass sdSPI(HSPI);
#define SD_CS       32 
#define SD_SCLK     18 
#define SD_MISO     19 
#define SD_MOSI     13
//////////////////////////////////////////////

////////////////////////////// Configure DS18B20 libraries ////////////////////////
#include <OneWire.h>
#include <DallasTemperature.h>
///////////////////////////////////////////////////////////////////////////////////
                    
int readingID = 0;

////////////////////////SWC: Soil Water Content Capacitive Module//////////////////////////////
const int ToneOutput = 25;                     // pin 25 is used for sending a 600 kHz PWM tone to the MoistSensor1, 2 and 3 
const int freq = 600000;
const int Channell = 1;
const int resolution = 8;
const int SWCPin1 = 12;                  // 12 bits ADC pin 12 senses the voltage level of the SWC1 (values 0 - 4095)
const int SWCPin2 = 14;                  // 12 bits ADC pin 14 senses the voltage level of the SWC1 (values 0 - 4095)
const int SWCPin3 = 15;                  // 12 bits ADC pin 15 senses the voltage level of the SWC1 (values 0 - 4095)
int SWC1 = 0;                            // Variable to store the water content for sensor 1 
float  SWC_volt1=0.0 ;
int SWC2 = 0;                            // Variable to store the water content for sensor 2 
float  SWC_volt2=0.0 ;
int SWC3 = 0;                            // Variable to store the water content for sensor 3 
float  SWC_volt3=0.0 ;
/////////////////////////////////////////////////////////////////////////////////////

////////////////////////MPX5100DP Module////////////////////////////////////////////
const int TensioPin1 = 34;   /* MPX5100DP sensor for tensiomerter#1*/          
const int TensioPin2 = 35;   /* MPX5100DP sensor for tensiomerter#2 */ 

int Tensio1 = 0;             // Variable to store MPX5100DP data for matric potential sensor 1 
float  Tensio_volt1=0.0 ;
int Tensio2 = 0;            // Variable to store MPX5100DP data for matric potential sensor 2 
float  Tensio_volt2=0.0 ;

 float Tensio_MPX5100DP_1;
 float Tensio_end_MPX5100DP_1;
 float Tensio_MPX5100DP_1_Offset;
 float Tensio_cm_water_MPX5100DP_1;
 
 float Tensio_MPX5100DP_2;
 float Tensio_end_MPX5100DP_2;
 float Tensio_MPX5100DP_2_Offset;
 float Tensio_cm_water_MPX5100DP_2;
 
/////////////////////////////////////////////////////////////////////////////////////

String dataMessage;

////////////////////////DS18B20 (Temperature) Module//////////////////////////////////
// Data wire is connected to ESP32 GPIO 0 
#define ONE_WIRE_BUS 33  
// Setup a oneWire instance to communicate with a OneWire device
OneWire oneWire(ONE_WIRE_BUS);
// Pass our oneWire reference to Dallas Temperature sensor 
DallasTemperature sensors(&oneWire);
float temperature;
/////////////////////////////////////////////////////////////////////////////////////

////////////////////////Battery Level//////////////////////////////////

float batteryLevel;
float batteryLevel_volts;
float batteryLevel_Perc;

/////////////////////////////////////////////////////////////////////////////////////

TwoWire I2CPower = TwoWire(0);

//TinyGSM Client for Internet connection
TinyGsmClient client(modem);

#define uS_TO_S_FACTOR 1000000ULL   /* Conversion factor for micro seconds to seconds */
#define TIME_TO_SLEEP 14400        //   14400 (i.e. 4 hours)     /* Time ESP32 will go to sleep (in seconds) 20 seconds */

#define IP5306_ADDR          0x75
#define IP5306_REG_SYS_CTL0  0x00

bool setPowerBoostKeepOn(int en){
  I2CPower.beginTransmission(IP5306_ADDR);
  I2CPower.write(IP5306_REG_SYS_CTL0);
  if (en) {
    I2CPower.write(0x37); // Set bit1: 1 enable 0 disable boost keep on
  } else {
    I2CPower.write(0x35); // 0x37 is default reg value
  }
  return I2CPower.endTransmission() == 0;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void setup() {
////////////////////////////////////////Initializing Sensors and Monitor (at baud rate of 115200)/////////////////////////////////////// 
   sensors.begin(); 
   SerialMon.begin(115200);
///////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////
  sdSPI.begin(SD_SCLK, SD_MISO, SD_MOSI, SD_CS);
 
//  Start I2C communication
  I2CPower.begin(I2C_SDA, I2C_SCL, 400000);

  // Keep power when running from battery
  bool isOk = setPowerBoostKeepOn(1);
  SerialMon.println(String("IP5306 KeepOn ") + (isOk ? "OK" : "FAIL"));

  // Set modem reset, enable, power pins/////////////
  pinMode(MODEM_PWKEY, OUTPUT);
  pinMode(MODEM_RST, OUTPUT);
  pinMode(MODEM_POWER_ON, OUTPUT);
  digitalWrite(MODEM_PWKEY, LOW);
  digitalWrite(MODEM_RST, HIGH);
  digitalWrite(MODEM_POWER_ON, HIGH);
////////////////////////////////////////////////////

  // Set GSM module baud rate and UART pins
  SerialAT.begin(115200, SERIAL_8N1, MODEM_RX, MODEM_TX);
  delay(3000);

  // Restart SIM800 module, it takes quite some time
  // To skip it, call init() instead of restart()
 
  SerialMon.println("Initializing modem...");
  modem.restart();
  // use modem.init() if you don't need the complete restart


 ////////////////////////////   SWC   ///////////////////////////
 ledcSetup(Channell, freq, resolution);         // configure the PWM functionalitites
 ledcAttachPin(ToneOutput, Channell);           // attach the channell  to the GPIO pin 25 for generating a PWM signal of 600kHz
  
  pinMode(SWCPin1, INPUT);            
  pinMode(SWCPin2, INPUT);
  pinMode(SWCPin3, INPUT);
 ////////////////////////////////////////////////////////////////

////////////////////////////   MPX5100DP   //////////////////////
  pinMode(TensioPin1, INPUT);              
  pinMode(TensioPin2, INPUT);  
/////////////////////////////////////////////////////////////////

  ////////////////////// Initialize SD card  ///////////////////
  SerialMon.print("Card Initialization: ");
                                                                                          
 SD.begin(SD_CS, sdSPI);  
 
if (!SD.begin(SD_CS, sdSPI)) {
  
    Serial.println("Card Mount Failed");
    return;
  }
  uint8_t cardType = SD.cardType();
  if(cardType == CARD_NONE) {
    SerialMon.println("No SD card attached");
    return;
  }
  SerialMon.println("Initializing SD card...");

 if (!SD.begin(SD_CS, sdSPI)) {
    SerialMon.println("ERROR - SD card initialization failed!");
    return;    // init failed
  }

  // If the data.txt file doesn't exist, create a file on the SD card and write the data labels
  File file = SD.open("/Sensor_Out.txt");
  if(!file) {
    SerialMon.println("File doens't exist");
    SerialMon.println("Creating file...");
  //  writeFile(SD, "/Sensor_Out.txt", "SWC1(volts)-SWC2(volts)-SWC3(volts)- MPX5100DP_1 (volts)- MPX5100DP_2 (volts) \r\n");
    writeFile(SD, "/Sensor_Out.txt", "sensor output \r\n");
  }
  else {
    SerialMon.println("File already exists");  
  }
  file.close();

  }
//////////////////////////////////////////////////////////////////////////////////////
void loop() {
////////////////////////////////////////////////////////////////
  // Configure the wake up source as timer wake up  
esp_sleep_enable_timer_wakeup(TIME_TO_SLEEP * uS_TO_S_FACTOR);
/////////////////////////////////////////////////////////////////
batteryLevel = map(analogRead(35), 0.0f, 4095.0f, 0, 100);
// /////////////////////////////////////////////////////////////////////////////
////  analogRead(SWCPin1); /* Insert a pause between two analogic measurements */ 
delay (1000);    
/////////////////////////////////////////////////////////////////////////////// 
batteryLevel_volts = analogRead(35)*1.7*(4.2 / 4096.0); // being 1.7 the conversion factor 
batteryLevel_Perc=(batteryLevel_volts/4.2)*100;
///////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
delay (200);    
///////////////////////////////////////////////////////////////////////////// 
  
  readingID++; // Increment readingID on every new reading
  
  //////////////////////////////////////Temperature Readings//////////////////////////////
  sensors.requestTemperatures(); 
  temperature = sensors.getTempCByIndex(0); // Temperature in Celsius
  delay(200);
   sensors.requestTemperatures(); 
  temperature = sensors.getTempCByIndex(0); // Temperature in Celsius
    
 //////////////////////////////////////////////////END TEMPERATURE Readings/////////////

///////////////////////TEMPERATURE (DS18B20 SENSOR) OUTPUT//////////////////////
  SerialMon.print("\n");
  SerialMon.print("Temperature: ");
  SerialMon.print(temperature);
  SerialMon.println(" °C");
////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
delay (200);    
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////SWC1 Readings/////////////////////////////////////
int counter_1 =0;
while (counter_1 <= 2) {
  counter_1=counter_1+1;
 Serial.println(counter_1);
SWC1= 0;
  ledcWrite(Channell, 128);                             // send a PWM signal of 600 kHz to pin 25 with a dutycycle of 50%
  delay(200);                                           // allow the circuit to stabilize  

  for (int m = 1; m < 50 ; m++)                          // take 50 consecutive measurements 
    {
      SWC1 = SWC1 + analogRead(SWCPin1) ;    // Read data from analog pin 4 and add it to MoistLevel1 variable
      delay (1000);
    }
    } //chiudi il ciclo while
   SWC1 = SWC1 / 50;                       // Determine the average of 5 measurements 
   SWC_volt1 = SWC1 * (3.3 / 4096.0);     //ADC convert 12 bit at 3.3 volts input
     ledcWrite(Channell, 0);                                // stop generating PWM tones at pin 25
//  } //chiudi il ciclo for: k  
/////////////////////////////////////End SWC1 Readings/////////////////////////////////////
delay (200);
 ///////////////////////////////////////SWC2 Readings/////////////////////////////////////
int counter_2 =0;
while (counter_2 <= 2) {
  counter_2=counter_2+1;
  Serial.println(counter_2);
SWC2= 0;
  ledcWrite(Channell, 128);                             // send a PWM signal of 600 kHz to pin 25 with a dutycycle of 50%
  delay(200);                                           // allow the circuit to stabilize  
  for (int m = 1; m < 50 ; m++)                          // take 50 consecutive measurements 
    {
      SWC2 = SWC2 + analogRead(SWCPin2) ;    // Read data from analog pin 4 and add it to MoistLevel1 variable
      delay (1000);
    }
    } //chiudi il ciclo while
   SWC2 = SWC2 / 50;                       // Determine the average of 5 measurements 
   SWC_volt2 = SWC2 * (3.3 / 4096.0);     //ADC convert 12 bit at 3.3 volts input
     ledcWrite(Channell, 0);                                // stop generating PWM tones at pin 25
/////////////////////////////////////End SWC2 Readings/////////////////////////////////////
delay (200); 
 ///////////////////////////////////////SWC3 Readings/////////////////////////////////////
int counter_3 =0;
while (counter_3 <= 2) {
  counter_3=counter_3+1;
  Serial.println(counter_3);
SWC3= 0;
  ledcWrite(Channell, 128);                             // send a PWM signal of 600 kHz to pin 25 with a dutycycle of 50%
  delay(200);                                           // allow the circuit to stabilize  
  for (int m = 1; m < 50 ; m++)                          // take 50 consecutive measurements 
    {
      SWC3 = SWC3 + analogRead(SWCPin3) ;    // Read data from analog pin 4 and add it to MoistLevel1 variable
      delay (1000);
    }
    } //chiudi il ciclo while
   SWC3 = SWC3 / 50;                       // Determine the average of 50 measurements 
   SWC_volt3 = SWC3 * (3.3 / 4096.0);     //ADC convert 12 bit at 3.3 volts input
     ledcWrite(Channell, 0);                                // stop generating PWM tones at pin 25
/////////////////////////////////////End SWC3 Readings////////////////////////////////
delay(1000);
//////////////////////Pressure tranducer#1 (MPX5100DP) sensor module///////////////////
Tensio1= 0;
 for (int m = 1; m < 50 ; m++)  {                        // take 50 consecutive measurements 
      
    Tensio1 = Tensio1 + analogRead(TensioPin1);
    delay(10);
  }
  
   Tensio1 = Tensio1 / 50;                       // Determine the average of 50 measurements 
   Tensio_volt1 = Tensio1 * (5.0/ 4096.0);     //ADC convert 12 bit at 5.0 volts input                                                       
////////////////////End Pressure tranducer#1 (MPX5100DP) sensor module/////////////////
delay (2000);
//////////////////////Pressure tranducer#2 (MPX5100DP) sensor module///////////////////
Tensio2= 0;
 for (int m = 1; m < 50 ; m++)  {                        // take 50 consecutive measurements 
      
    Tensio2 = Tensio2 + analogRead(TensioPin2);
    delay(10);
  }
  
   Tensio2 = Tensio2 / 50;                       // Determine the average of 50 measurements 
   Tensio_volt2 = Tensio2 * (5.0/ 4096.0);     //ADC convert 12 bit at 5.0 volts input                                                       
////////////////////End Pressure tranducer#1 (MPX5100DP) sensor module/////////////////


/////////////////////////////////////////////////////////////////////////////
delay (200);  
/////////////////////////////////////////////////////////////////////////////

//////////////////////////Calibration Functions for the different probe sensors/////////////////////////////////////////////////

/////////////////// Pressure Transducer#1 (MPX5100DP) calibration function module: Optional////////////////////////////////////////////////

//  Tensio_MPX5100DP_1_Offset = 0; /* Set the instrumental offset for the Pressure Trasducer#1 calibration function */
// 
//  Tensio_MPX5100DP_1 = 22.22*Tensio_volt1-4.44; /* Pressure Transducer#1 (MPX5100DP) calibration function (kPa) */
//  Tensio_end_MPX5100DP_1 = Tensio_MPX5100DP_1-Tensio_MPX5100DP_1_Offset;
//  Tensio_cm_water_MPX5100DP_1 = (Tensio_end_MPX5100DP_1*10.1972); /* Convert to cm water column */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
///////////////////// Pressure Transducer#2 (MPX5100DP) calibration function module: Optional////////////////////////////////////////////////
// 
// Tensio_MPX5100DP_2_Offset = 0; /* Set the instrumental offset for the Pressure Trasducer#2 calibration function */
// 
//  Tensio_MPX5100DP_2 = 22.22*Tensio_volt2-4.44; /* Pressure Transducer#2 (MPX5100DP) calibration function (kPa) */
//  Tensio_end_MPX5100DP_2 = Tensio_MPX5100DP_2-Tensio_MPX5100DP_2_Offset;
//  Tensio_cm_water_MPX5100DP_2 = (Tensio_end_MPX5100DP_2*10.1972); /* Convert to cm water column */
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//////////////Capacitive sensor calibration function module for volumetric water content determination: Optional/////////////////////////////
// teta_Top_Ring = 0.0147*pow(voltage1,2)-0.1565*voltage1+0.4411; //Put here your calibration function (soil dependent)
//
// teta_Middle_Ring = 0.0147*pow(voltage2,2)-0.1565*voltage2+0.4411; //Put here your calibration function (soil dependent)
//
// teta_Bottom_Ring = 0.0147*pow(voltage3,2)-0.1565*voltage3+0.4411; //Put here your calibration function (soil dependent)
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



  ////////////////////////////////////////////////////OUTPUT INITIALIZATION//////////////////////////

///////////////SWC SENSORS OUTPUT///////////////
  ////////TOP RING
  SerialMon.print("\n");
  SerialMon.print(" Avg. value SWC1 = ");
  SerialMon.print(SWC1);
  SerialMon.print("\n");
  SerialMon.print(" volts SWC1 = ");
  SerialMon.print(SWC_volt1);
  SerialMon.print("\n");
  ////////MIDDLE RING
  SerialMon.print("\n");
  SerialMon.print(" Avg. value SWC2 = ");
  SerialMon.print(SWC2);
  SerialMon.print("\n");
  SerialMon.print(" volts SWC2 = ");
  SerialMon.print(SWC_volt2);
  SerialMon.print("\n");
  ////////BOTTOM RING
  SerialMon.print("\n");
  SerialMon.print(" Avg. value SWC3 = ");
  SerialMon.print(SWC3);
  SerialMon.print("\n");
  SerialMon.print(" volts SWC3= ");
  SerialMon.print(SWC_volt3);
  SerialMon.print("\n");

   ///////////////TENSIOMETER#1 OUTPUT/////////////// 
  SerialMon.print("\n");
  SerialMon.print(" Avg. value Tensio1 = ");
  SerialMon.print(Tensio1);
  SerialMon.print("\n");
  SerialMon.print(" volts Tensio1 = ");
  SerialMon.print(Tensio_volt1);
  SerialMon.print("\n");
//  SerialMon.print(" Matric potential (kPa) = "); //optional
//  SerialMon.print(Tensio_end_MPX5100DP_1);
//  SerialMon.print("\n");
  ///////////////TENSIOMETER#2 OUTPUT/////////////// 
  SerialMon.print("\n");
  SerialMon.print(" Avg. value Tensio2= ");
  SerialMon.print(Tensio2);
  SerialMon.print("\n");
  SerialMon.print(" volts Tensio2= ");
  SerialMon.print(Tensio_volt2);
  SerialMon.print("\n");
//  SerialMon.print(" Matric potential (kPa) = "); //optional
//  SerialMon.print(Tensio_end_MPX5100DP_2);
//  SerialMon.print("\n");
  ////////////////////////////////////////////////
  
  ///////////////////////TEMPERATURE (DS18B20 SENSOR) OUTPUT//////////////////////
  SerialMon.print("\n");
  SerialMon.print("Temperature: ");
  SerialMon.print(temperature);
  SerialMon.println(" °C");
///////////////////////////Battery Level: Optional///////////////////////////////////////////
//  SerialMon.print("\n");
//  SerialMon.print("Battery Level: ");
//  SerialMon.print(batteryLevel);
//  SerialMon.println(" %");
//   SerialMon.print(batteryLevel_volts);
//  SerialMon.println("volts");

////////////////////SD CARD  OUTPUT//////////////////////////////////
// Write the sensor readings (i.e. datamessage) on the SD card
  
 dataMessage = String(counter_1)  +  String(SWC1)+ ","  + String(SWC_volt1) + "," + String(SWC2)+ "," + String(SWC_volt2) + "," + String(SWC3)+ "," + String(SWC_volt3)  + "," + String(Tensio1) +  "," +   String(Tensio_volt1) +  "," +  String(Tensio_end_MPX5100DP_1) +  ","
  + String(Tensio2) +  ","  + String(Tensio_volt2)+  "," +  String(Tensio_end_MPX5100DP_2) + "," +  String(temperature) +  "\r\n";
  SerialMon.print("Save data: ");
  SerialMon.println(dataMessage);
  appendFile(SD, "/Sensor_Out.txt", dataMessage.c_str());
///////////////////End SD Card////////////////////////////////////// 

/////////////////////////////////////////////Begin GPRS connection (DON'T MODIFY)//////////////////////////////////////////////////
  SerialMon.print("Connecting to APN: ");
    SerialMon.print(apn);
  if (!modem.gprsConnect(apn, gprsUser, gprsPass)) {
 
    SerialMon.println(" fail");
  }
  else {
    SerialMon.println(" OK");
    
    SerialMon.print("Connecting to ");
    SerialMon.print(server);
    if (!client.connect(server, port)) {
      SerialMon.println(" fail");
    }
    else {
      SerialMon.println(" OK");
    
      // Making an HTTP POST request
      SerialMon.println("Performing HTTP POST request...");
      // Prepare your HTTP POST request data (Temperature in Celsius degrees)

      String httpRequestData = "api_key=" + apiKeyValue + "&field1=" + String(SWC_volt1) + "&field2="+ String(SWC_volt2)+ "&field3=" + String(SWC_volt3) + "&field4=" + String(temperature) 
     + "&field5=" + String(Tensio_volt1)+ "&field6=" + String(Tensio_volt2)+ "&field7=" + String(batteryLevel_Perc)+ "&field8=" + String(batteryLevel_volts)+  ""; 
     
      client.print(String("POST ") + resource + " HTTP/1.1\r\n");
      client.print(String("Host: ") + server + "\r\n");
      client.println("Connection: close");
      client.println("Content-Type: application/x-www-form-urlencoded");
      client.print("Content-Length: ");
      client.println(httpRequestData.length());
      client.println();
      client.println(httpRequestData);
 
      unsigned long timeout = millis();
      while (client.connected() && millis() - timeout < 10000L) {
        // Print available data (HTTP response from server)
        while (client.available()) {
          char c = client.read();
          SerialMon.print(c);
          timeout = millis();
        }
      }
      SerialMon.println();
    
      // Close client and disconnect
      client.stop();
      SerialMon.println(F("Server disconnected"));
      modem.gprsDisconnect();
      SerialMon.println(F("GPRS disconnected"));
    }
  }
/////////////////////////////////////////////////////////////////////////////////////////////
 
// Put ESP32 into deep sleep mode (with timer wake up)
  esp_deep_sleep_start();
}            //END VOID LOOP

//// Write to the SD card (DON'T MODIFY)
void writeFile(fs::FS &fs, const char * path, const char * message) {
  SerialMon.printf("Writing file: %s\n", path);

  File file = fs.open(path, FILE_WRITE);
  if(!file) {
    SerialMon.println("Failed to open file for writing");
    return;
  }
  if(file.print(message)) {
    SerialMon.println("File written");
  } else {
    SerialMon.println("Write failed");
  }
  file.close();
}

/////////////////////////// Append data to the SD card ////////////////////////////
void appendFile(fs::FS &fs, const char * path, const char * message) {
  SerialMon.printf("Appending to file: %s\n", path);

  File file = fs.open(path, FILE_APPEND);
  if(!file) {
    SerialMon.println("Failed to open file for appending");
    return;
  }
  if(file.print(message)) {
    SerialMon.println("Message appended");
  } else {
    SerialMon.println("Append failed");
  }
  file.close();
}
/////////////////////////////////     END    /////////////////////////////////////
